Attribute VB_Name = "modFunctions"
Private m_strConnectionString    As String ' Module Level Variable to hold the
                                           ' Connection String Information

Public Function ExportToXML(ByVal str_XMLFileName As String, _
                            ByVal str_SheetName As String) As Boolean

'===========================================================================
' AIM   :   This function Exports the selected Excel sheet out to XML
'           format. To do this we first open the excel sheet based on
'           connection string. Using ADO 2.5 , we can open the excel
'           file and query the nominated sheet. Once executed, returns
'           an ADO Recordset result.
'
' Arguments :   str_XMLFileName         String
'               This is the filename and path of the xml file to be saved.
'
'               str_SheetName           String
'               Used to pass in the Sheet you want to query
'
'===========================================================================


On Error GoTo ExportToXMLErr

Dim cnnExcel    As ADODB.Connection
Dim rstData     As ADODB.Recordset

' Create a new ADO Connection Object
Set cnnExcel = New ADODB.Connection
' Set the Connection String
cnnExcel.ConnectionString = m_strConnectionString
' Set the Cursor location to Client Side
cnnExcel.CursorLocation = adUseClient
' Open the Connection
cnnExcel.Open
' Obtain the recordset by executing SQL statement with sheet name
Set rstData = cnnExcel.Execute("SELECT * FROM [" & str_SheetName & "]")
' If records were returned the save the file
If rstData.RecordCount > 0 Then
    rstData.Save str_XMLFileName, adPersistXML
    ' Set to successful
    ExportToXML = True
Else
    ' NO records found advise user and return false
    MsgBox "No Data to Save.", vbExclamation
    ExportToXML = False
End If

' Close the Recordset and connection
rstData.Close
cnnExcel.Close

    
ExportToXMLExit:
    ' Release objects
    Set rstData = Nothing
    Set cnnExcel = Nothing
    Exit Function

ExportToXMLErr:
    ExportToXML = False
    ErrHandler
    Resume ExportToXMLExit
End Function

Public Function OpenFile(ByVal lng_FormHwnd As Long, ByVal bln_ShowOpen As Boolean) As String


On Error GoTo OpenFileErr

' =======================================================================
' AIM   :   Opens a dialog box based on the bln_ShowOpen argument.
'           This function creates a browse object to open the dialog box
'
' Arguments :   lng_FormHwnd        Long
'               This is used to bind the dialog box to the requesting
'               form handle.
'
'               bln_ShowOpen        Boolean
'               Used to determine if you are showing the Open or Save
'               Dialog Box. True = Open ; False = Save
'
' =======================================================================

    Dim objBrowse   As clsDialogBox
    Dim strFilter   As String
    
    ' Set the filter options for the dialog box
    strFilter = "Excel Files (*.xls) " & Chr(0) & "*.xls" & Chr(0) & _
                "XML Files (*.xml) " & Chr(0) & "*.xml" & Chr(0) & _
                "All Files (*.*) " & Chr(0) & "*.*" & Chr(0)
                
    ' Create the Dialogbox Object
    Set objBrowse = New clsDialogBox
    ' Set the filter options
    objBrowse.Filter = strFilter
    ' Determine if the Open or Save was chosen
    If bln_ShowOpen Then
        objBrowse.DialogTitle = "Open Excel File"
        OpenFile = objBrowse.ShowOpen(lng_FormHwnd)
    Else
        objBrowse.DialogTitle = "Save XML File"
        OpenFile = objBrowse.ShowSave(lng_FormHwnd)
    End If
        
OpenFileExit:
    ' Release Object
    Set objBrowse = Nothing
    Exit Function

OpenFileErr:
    ErrHandler
    Resume OpenFileExit
End Function

Public Function ApplyExtension(ByVal str_FileName As String, ByVal str_Extension As String) As String

' ================================================================
' Aim : Applies an extension to a filename
'
'   Arguments   :   str_Filename        String
'                   This is the Filename to be verified
'
'                   str_Extension       String
'                   The extension name to verifiy aganist and
'                   apply to the filename
'
'
' ================================================================

On Error GoTo ApplyExtensionErr

Dim intFind As Integer

intFind = InStr(1, str_FileName, ".", vbTextCompare)
If intFind > 0 Then
    ApplyExtension = Left(str_FileName, intFind) & str_Extension
Else
    ApplyExtension = str_FileName & "." & str_Extension
End If

ApplyExtensionExit:
    Exit Function

ApplyExtensionErr:
    ErrHandler
End Function

Public Function ReadSchema(ByVal str_ExcelFileName As String) As String()

' ==============================================================
' AIM   :   To Open the Excel file and Read it's Schema.
'           Then to locate the Sheet names and return an
'           array result set.
'
' Arguments :   str_ExcelFilename       String
'               Used to Obtain the Excel file to Open
'
' ===============================================================

On Error GoTo ReadSchemaErr

Dim cnnExcel            As ADODB.Connection
Dim rstData             As ADODB.Recordset
Dim astrSheets()        As String
Dim intSheet            As Integer

' Sheet the Sheet Count to 1 as Default
intSheet = 1

' Create a ADO Connection
Set cnnExcel = New ADODB.Connection
' Set the connection string info
cnnExcel.ConnectionString = m_strConnectionString
' Set cursor location to client side
cnnExcel.CursorLocation = adUseClient
' Open the Excel sheet
cnnExcel.Open
' Return the Excel Columns schema in a Recordset
Set rstData = cnnExcel.OpenSchema(adSchemaColumns)
    
' Loop through the recordset to obatin
' the Sheet names
Do Until rstData.EOF
    ' For each sheetname there is a $ assigned
    ' We used this indecator to capture the
    ' sheetname.
    If Right(rstData!TABLE_NAME, 1) = "$" Then
        ' Make sure that the Sheetname hasn't been
        ' already applied to the array.
        If IsUnquie(astrSheets, rstData!TABLE_NAME) Then
            ' If unquie then Add the Sheet name to
            ' the array and preserve the previous
            ' data in the array.
            ReDim Preserve astrSheets(intSheet)
            astrSheets(intSheet) = rstData!TABLE_NAME
            ' Increase the Sheet Counter
            intSheet = intSheet + 1
        End If
    End If
    rstData.MoveNext
Loop

' Return the Array back to the function
ReadSchema = astrSheets

ReadSchemaExit:
    ' Release Objects
    Set cnnExcel = Nothing
    Set rstData = Nothing
    Exit Function

ReadSchemaErr:
    ErrHandler
End Function

Public Sub SetConnection(ByVal str_ExcelFileName As String)

' ======================================================
' AIM   :   To Set the ADO Connection String Variable
'
' Arguments :   str_ExcelFileName       String
'               Used to set the Excel filename into the
'               connection string.
 ' ======================================================

 m_strConnectionString = "Provider=MSDASQL.1;Persist Security " & _
                         "Info=False;Extended Properties=" & Chr(34) & _
                         "DSN=Excel Files;DBQ=" & str_ExcelFileName & ";" & _
                         "DefaultDir=c:;DriverId=790;" & _
                         "MaxBufferSize=2048;PageTimeout=5;" & Chr(34)
                         
End Sub

Private Function IsUnquie(ByRef a_strArray() As String, var_Value As Variant) As Boolean

' ========================================================
' AIM   :   To Determine if a value is unquie in an array.
'
' Arguments :   a_strArray()        String
'               The array that needs to be tested
'
'               var_Value           Variant
'               The value that is tested against the Array
'
' ========================================================


On Error GoTo IsUnquieErr

Dim intLoop As Integer

' By Default the Value is unquie
IsUnquie = True
' Loop through the Array and make sure
' that the value is not on the array.
For intLoop = 0 To UBound(a_strArray)
    If a_strArray(intLoop) = var_Value Then
        ' If the value is in the
        ' array, then return false
        ' Exit the Loop
        IsUnquie = False
        Exit For
    End If
Next intLoop

IsUnquieExit:
    Exit Function

IsUnquieErr:
    ' Handle any Errors : 9 is a subscription out of Range
    ' May apear at some time
    If Err.Number <> 9 Then
        ErrHandler
    End If
End Function

Public Sub ErrPushStack(ByVal str_Function As String)
' This function Is here to dummy the Classes Error handling.
' Our Methodology uses an Error Handler Module (Not Included in the Project)
' To Stack / Pop Errors and send the Errors off to an E-mail address..
End Sub
Public Sub ErrPopStack()
' This function Is here to dummy the Classes Error handling.
' Our Methodology uses an Error Handler Module (Not Included in the Project)
' To Stack / Pop Errors and send the Errors off to an E-mail address..
End Sub

Public Function ErrHandler()
    ' Basic Error Handling Skills here ....... I Know
    MsgBox Err.Number & ": " & Err.Description
End Function
